@@ -55,7 +55,7 @@ def get_safe_mode():
 
 def running_under_pytest():
     """
-    Return True if currently running under py.test.
+    Return True if currently running under pytest.
 
     This function is used to do some adjustment for testing. The environment
     variable SPYDER_PYTEST is defined in conftest.py.
@@ -63,6 +63,11 @@ def running_under_pytest():
     return bool(os.environ.get('SPYDER_PYTEST'))
 
 
+def running_in_ci():
+    """Return True if currently running under CI."""
+    return bool(os.environ.get('CI'))
+
+
 def is_stable_version(version):
     """
     Return true if version is stable, i.e. with letters in the final component.
