@@ -20,9 +20,9 @@ from qtpy.QtWidgets import (QApplication, QDialog, QDialogButtonBox,
                             QInputDialog, QMessageBox, QTextEdit)
 
 # Local imports
-from spyder.plugins.explorer.widgets.main_widget import (
-    FileExplorerTest, ProjectExplorerTest)
-from spyder.plugins.projects.widgets.explorer import (
+from spyder.plugins.explorer.widgets.main_widget import FileExplorerTest
+from spyder.plugins.projects.widgets.main_widget import ProjectExplorerTest
+from spyder.plugins.projects.widgets.main_widget import (
     ProjectExplorerTest as ProjectExplorerTest2)
 
 
@@ -229,7 +229,7 @@ def test_single_click_to_open(qtbot, file_explorer):
     run_test_helper(single_click=False, initial_index=initial_index)
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_get_common_file_associations(qtbot, file_explorer_associations):
     widget = file_explorer_associations.explorer.treewidget
     associations = widget.get_common_file_associations(
@@ -247,7 +247,7 @@ def test_get_common_file_associations(qtbot, file_explorer_associations):
     assert associations[0][-1] == '/some/fake/some_app_1' + ext
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_get_file_associations(qtbot, file_explorer_associations):
     widget = file_explorer_associations.explorer.treewidget
     associations = widget.get_file_associations('/some/path/file.txt')
@@ -260,7 +260,7 @@ def test_get_file_associations(qtbot, file_explorer_associations):
     assert associations[0][-1] == '/some/fake/some_app_1' + ext
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_create_file_manager_actions(qtbot, file_explorer_associations,
                                      tmp_path):
     widget = file_explorer_associations.explorer.treewidget
@@ -290,7 +290,7 @@ def test_create_file_manager_actions(qtbot, file_explorer_associations,
     assert not action_texts
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_clicked(qtbot, file_explorer_associations, tmp_path):
     widget = file_explorer_associations.explorer.treewidget
     some_dir = tmp_path / 'some_dir'
@@ -322,7 +322,7 @@ def test_clicked(qtbot, file_explorer_associations, tmp_path):
     qtbot.keyClick(widget, Qt.Key_Return)
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_check_launch_error_codes(qtbot, file_explorer_associations):
     widget = file_explorer_associations.explorer.treewidget
 
@@ -353,7 +353,7 @@ def test_check_launch_error_codes(qtbot, file_explorer_associations):
     assert not res
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_open_association(qtbot, file_explorer_associations, tmp_path):
     widget = file_explorer_associations.explorer.treewidget
     some_dir = tmp_path / 'some_dir'
@@ -373,7 +373,7 @@ def test_open_association(qtbot, file_explorer_associations, tmp_path):
     widget.open_association('some-app')
 
 
-@pytest.mark.first
+@pytest.mark.order(1)
 def test_update_filters(file_explorer, qtbot):
     """
     Test that view is updated if the filter button is activated and
