@@ -17,19 +17,27 @@ from textwrap import dedent
 # Third party imports
 from qtpy.QtCore import Signal, QThread
 from qtpy.QtWidgets import QMessageBox
+from qtpy import QtCore, QtWidgets, QtGui
 
 # Local imports
-from spyder.config.base import _, running_under_pytest
-from spyder.config.manager import CONF
+from spyder.config.base import (
+    _, is_pynsist, running_in_mac_app, running_under_pytest)
 from spyder.py3compat import to_text_string
-from spyder.utils import programs, encoding
+from spyder.utils.palette import SpyderPalette
+from spyder.utils import encoding
+from spyder.utils.clipboard_helper import CLIPBOARD_HELPER
 from spyder.utils import syntaxhighlighters as sh
-from spyder.plugins.ipythonconsole.utils.style import create_qss_style, create_style_class
+from spyder.plugins.ipythonconsole.utils.style import (
+    create_qss_style, create_style_class)
 from spyder.widgets.helperwidgets import MessageCheckBox
 from spyder.plugins.ipythonconsole.comms.kernelcomm import KernelComm
 from spyder.plugins.ipythonconsole.widgets import (
-        ControlWidget, DebuggingWidget, FigureBrowserWidget,
-        HelpWidget, NamepaceBrowserWidget, PageControlWidget)
+    ControlWidget, DebuggingWidget, FigureBrowserWidget, HelpWidget,
+    NamepaceBrowserWidget, PageControlWidget)
+
+
+MODULES_FAQ_URL = (
+    "https://docs.spyder-ide.org/5/faq.html#using-packages-installer")
 
 
 class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
@@ -51,11 +59,11 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
 
     # For DebuggingWidget
     sig_pdb_step = Signal(str, int)
-    sig_pdb_state = Signal(bool, dict)
+    sig_pdb_state_changed = Signal(bool, dict)
     sig_pdb_prompt_ready = Signal()
 
     # For ShellWidget
-    focus_changed = Signal()
+    sig_focus_changed = Signal()
     new_client = Signal()
     sig_is_spykernel = Signal(object)
     sig_kernel_restarted_message = Signal(str)
@@ -64,7 +72,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
     sig_remote_execute = Signal()
 
     # For global working directory
-    sig_change_cwd = Signal(str)
+    sig_working_directory_changed = Signal(str)
 
     # For printing internal errors
     sig_exception_occurred = Signal(dict)
@@ -122,6 +130,10 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         # Internal kernel are always spyder kernels
         self._is_spyder_kernel = not external_kernel
 
+        # Show a message in our installers to explain users how to use
+        # modules that don't come with them.
+        self.show_modules_message = is_pynsist() or running_in_mac_app()
+
     def __del__(self):
         """Avoid destroying shutdown_thread."""
         if (self.shutdown_thread is not None
@@ -273,7 +285,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
     def remote_set_cwd(self, cwd):
         """Get current working directory from kernel."""
         self._cwd = cwd
-        self.sig_change_cwd.emit(self._cwd)
+        self.sig_working_directory_changed.emit(self._cwd)
 
     def set_bracket_matcher_color_scheme(self, color_scheme):
         """Set color scheme for matched parentheses."""
@@ -397,8 +409,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         # Pylab additions
         pylab_o = self.additional_options['pylab']
         autoload_pylab_o = self.additional_options['autoload_pylab']
-        mpl_installed = programs.is_module_installed('matplotlib')
-        if mpl_installed and (pylab_o and autoload_pylab_o):
+        if pylab_o and autoload_pylab_o:
             pylab_message = ("\nPopulating the interactive namespace from "
                              "numpy and matplotlib\n")
             banner = banner + pylab_message
@@ -439,19 +450,67 @@ the sympy module (e.g. plot)
         self._reading = False
 
     def _reset_namespace(self):
-        warning = CONF.get('ipython_console', 'show_reset_namespace_warning')
+        warning = self.get_conf('show_reset_namespace_warning')
         self.reset_namespace(warning=warning)
 
     def reset_namespace(self, warning=False, message=False):
         """Reset the namespace by removing all names defined by the user."""
-        reset_str = _("Remove all variables")
-        warn_str = _("All user-defined variables will be removed. "
-                     "Are you sure you want to proceed?")
-
         # Don't show the warning when running our tests.
         if running_under_pytest():
             warning = False
 
+        if warning:
+            reset_str = _("Remove all variables")
+            warn_str = _("All user-defined variables will be removed. "
+                         "Are you sure you want to proceed?")
+            box = MessageCheckBox(icon=QMessageBox.Warning, parent=self)
+            box.setWindowTitle(reset_str)
+            box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
+            box.setDefaultButton(QMessageBox.Yes)
+
+            box.set_checkbox_text(_("Don't show again."))
+            box.set_checked(False)
+            box.set_check_visible(True)
+            box.setText(warn_str)
+
+            box.buttonClicked.connect(
+                lambda button: self.handle_reset_message_answer(
+                    box, button, message)
+            )
+            box.show()
+        else:
+            self._perform_reset(message)
+
+    def handle_reset_message_answer(self, message_box, button, message):
+        """
+        Handle the answer of the reset namespace message box.
+
+        Parameters
+        ----------
+        message_box
+            Instance of the message box shown to the user.
+        button: QPushButton
+            Instance of the button clicked by the user on the dialog.
+        message: bool
+            Whether to show a message in the console telling users the
+            namespace was reset.
+        """
+        if message_box.buttonRole(button) == QMessageBox.YesRole:
+            self._update_reset_options(message_box)
+            self._perform_reset(message)
+        else:
+            self._update_reset_options(message_box)
+
+    def _perform_reset(self, message):
+        """
+        Perform the reset namespace operation.
+
+        Parameters
+        ----------
+        message: bool
+            Whether to show a message in the console telling users the
+            namespace was reset.
+        """
         # This is necessary to make resetting variables work in external
         # kernels.
         # See spyder-ide/spyder#9505.
@@ -460,36 +519,17 @@ the sympy module (e.g. plot)
         except AttributeError:
             kernel_env = {}
 
-        if warning:
-            box = MessageCheckBox(icon=QMessageBox.Warning, parent=self)
-            box.setWindowTitle(reset_str)
-            box.set_checkbox_text(_("Don't show again."))
-            box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
-            box.setDefaultButton(QMessageBox.Yes)
-
-            box.set_checked(False)
-            box.set_check_visible(True)
-            box.setText(warn_str)
-
-            answer = box.exec_()
-
-            # Update checkbox based on user interaction
-            CONF.set('ipython_console', 'show_reset_namespace_warning',
-                     not box.is_checked())
-            self.ipyclient.reset_warning = not box.is_checked()
-
-            if answer != QMessageBox.Yes:
-                return
-
         try:
             if self.is_waiting_pdb_input():
                 self.execute('%reset -f')
             else:
                 if message:
                     self.reset()
-                    self._append_html(_("<br><br>Removing all variables..."
-                                        "\n<hr>"),
-                                      before_prompt=False)
+                    self._append_html(
+                        _("<br><br>Removing all variables...<br>"),
+                        before_prompt=False
+                    )
+                    self.insert_horizontal_ruler()
                 self.silent_execute("%reset -f")
                 if kernel_env.get('SPY_AUTOLOAD_PYLAB_O') == 'True':
                     self.silent_execute("from pylab import *")
@@ -515,51 +555,62 @@ the sympy module (e.g. plot)
         except AttributeError:
             pass
 
+    def _update_reset_options(self, message_box):
+        """
+        Update options and variables based on the interaction in the
+        reset warning message box shown to the user.
+        """
+        self.set_conf(
+            'show_reset_namespace_warning',
+            not message_box.is_checked()
+        )
+        self.ipyclient.reset_warning = not message_box.is_checked()
+
     def create_shortcuts(self):
         """Create shortcuts for ipyconsole."""
-        inspect = CONF.config_shortcut(
+        inspect = self.config_shortcut(
             self._control.inspect_current_object,
             context='Console',
             name='Inspect current object',
             parent=self)
 
-        clear_console = CONF.config_shortcut(
+        clear_console = self.config_shortcut(
             self.clear_console,
             context='Console',
             name='Clear shell',
             parent=self)
 
-        restart_kernel = CONF.config_shortcut(
+        restart_kernel = self.config_shortcut(
             self.ipyclient.restart_kernel,
             context='ipython_console',
             name='Restart kernel',
             parent=self)
 
-        new_tab = CONF.config_shortcut(
+        new_tab = self.config_shortcut(
             lambda: self.new_client.emit(),
             context='ipython_console',
             name='new tab',
             parent=self)
 
-        reset_namespace = CONF.config_shortcut(
+        reset_namespace = self.config_shortcut(
             lambda: self._reset_namespace(),
             context='ipython_console',
             name='reset namespace',
             parent=self)
 
-        array_inline = CONF.config_shortcut(
+        array_inline = self.config_shortcut(
             self._control.enter_array_inline,
             context='array_builder',
             name='enter array inline',
             parent=self)
 
-        array_table = CONF.config_shortcut(
+        array_table = self.config_shortcut(
             self._control.enter_array_table,
             context='array_builder',
             name='enter array table',
             parent=self)
 
-        clear_line = CONF.config_shortcut(
+        clear_line = self.config_shortcut(
             self.ipyclient.clear_line,
             context='console',
             name='clear line',
@@ -667,11 +718,59 @@ the sympy module (e.g. plot)
 
         Fixes spyder-ide/spyder#4002.
         """
-        if command.startswith('%matplotlib') and \
-          len(command.splitlines()) == 1:
+        if (command.startswith('%matplotlib') and
+                len(command.splitlines()) == 1):
             if not 'inline' in command:
                 self.silent_execute(command)
 
+    def append_html_message(self, html, before_prompt=False,
+                            msg_type='warning'):
+        """
+        Append an html message enclosed in a box.
+
+        Parameters
+        ----------
+        before_prompt: bool
+            Whether to add the message before the next prompt.
+        msg_type: str
+            Type of message to be showm. Possible values are
+            'warning' and 'error'.
+        """
+        # The message is displayed in a table with a single cell.
+        table_properties = (
+            "border='0.5'" +
+            "width='90%'" +
+            "cellpadding='8'" +
+            "cellspacing='0'"
+        )
+
+        if msg_type == 'error':
+            header = _("Error")
+            bgcolor = SpyderPalette.COLOR_ERROR_2
+        else:
+            header = _("Warning")
+            bgcolor = SpyderPalette.COLOR_WARN_1
+
+        self._append_html(
+            f"<div align='center'><table {table_properties}>" +
+            f"<tr><th bgcolor='{bgcolor}'>{header}</th></tr>" +
+            "<tr><td>" + html + "</td></tr>" +
+            "</table></div>",
+            before_prompt=before_prompt
+        )
+
+    def insert_horizontal_ruler(self):
+        """
+        Insert a horizontal ruler at the current cursor position.
+
+        Notes
+        -----
+        This only works when adding a single horizontal line to a
+        message. For more complex messages, please use
+        append_html_message.
+        """
+        self._control.insert_horizontal_ruler()
+
     # ---- Spyder-kernels methods ---------------------------------------------
     def get_editor(self, filename):
         """Get editor for filename and set it as the current editor."""
@@ -703,7 +802,8 @@ the sympy module (e.g. plot)
         Bytes are returned instead of str to support non utf-8 files.
         """
         editorstack = self.get_editorstack()
-        if save_all and CONF.get('editor', 'save_all_before_run', True):
+        if save_all and self.get_conf(
+                'save_all_before_run', default=True, section='editor'):
             editorstack.save_all(save_new_files=False)
         editor = self.get_editor(filename)
 
@@ -750,6 +850,106 @@ the sympy module (e.g. plot)
         """Reimplemented to call our own restart mechanism."""
         self.ipyclient.restart_kernel()
 
+    def adjust_indentation(self, line, indent_adjustment):
+        """Adjust indentation."""
+        if indent_adjustment == 0 or line == "":
+            return line
+
+        if indent_adjustment > 0:
+            return ' ' * indent_adjustment + line
+
+        max_indent = CLIPBOARD_HELPER.get_line_indentation(line)
+        indent_adjustment = min(max_indent, -indent_adjustment)
+
+        return line[indent_adjustment:]
+
+    def paste(self, mode=QtGui.QClipboard.Clipboard):
+        """ Paste the contents of the clipboard into the input region.
+
+        Parameters
+        ----------
+        mode : QClipboard::Mode, optional [default QClipboard::Clipboard]
+
+            Controls which part of the system clipboard is used. This can be
+            used to access the selection clipboard in X11 and the Find buffer
+            in Mac OS. By default, the regular clipboard is used.
+        """
+        if self._control.textInteractionFlags() & QtCore.Qt.TextEditable:
+            # Make sure the paste is safe.
+            self._keep_cursor_in_buffer()
+            cursor = self._control.textCursor()
+
+            # Remove any trailing newline, which confuses the GUI and forces
+            # the user to backspace.
+            text = QtWidgets.QApplication.clipboard().text(mode).rstrip()
+
+            # Adjust indentation of multilines pastes
+            if len(text.splitlines()) > 1:
+                lines_adjustment = CLIPBOARD_HELPER.remaining_lines_adjustment(
+                    self._get_preceding_text())
+                eol_chars = "\n"
+                first_line, *remaining_lines = (text + eol_chars).splitlines()
+                remaining_lines = [
+                    self.adjust_indentation(line, lines_adjustment)
+                    for line in remaining_lines]
+                text = eol_chars.join([first_line, *remaining_lines])
+
+            # dedent removes "common leading whitespace" but to preserve
+            # relative indent of multiline code, we have to compensate for any
+            # leading space on the first line, if we're pasting into
+            # an indented position.
+            cursor_offset = cursor.position() - self._get_line_start_pos()
+            if text.startswith(' ' * cursor_offset):
+                text = text[cursor_offset:]
+
+            self._insert_plain_text_into_buffer(cursor, dedent(text))
+
+    def _get_preceding_text(self):
+        """Get preciding text."""
+        cursor = self._control.textCursor()
+        text = cursor.selection().toPlainText()
+        if text == "":
+            return ""
+        first_line_selection = text.splitlines()[0]
+        cursor.setPosition(cursor.selectionStart())
+        cursor.setPosition(cursor.block().position(),
+                           QtGui.QTextCursor.KeepAnchor)
+        preceding_text = cursor.selection().toPlainText()
+        first_line = preceding_text + first_line_selection
+        len_with_prompt = len(first_line)
+        # Remove prompt
+        first_line = self._highlighter.transform_classic_prompt(first_line)
+        first_line = self._highlighter.transform_ipy_prompt(first_line)
+
+        prompt_len = len_with_prompt - len(first_line)
+        if prompt_len >= len(preceding_text):
+            return ""
+
+        return preceding_text[prompt_len:]
+
+    def _save_clipboard_indentation(self):
+        """
+        Save the indentation corresponding to the clipboard data.
+
+        Must be called right after copying.
+        """
+        CLIPBOARD_HELPER.save_indentation(self._get_preceding_text(), 4)
+
+    def copy(self):
+        """
+        Copy the currently selected text to the clipboard.
+        """
+        super().copy()
+        self._save_clipboard_indentation()
+
+    def cut(self):
+        """
+        Copy the currently selected text to the clipboard and delete it
+        if it's inside the input buffer.
+        """
+        super().cut()
+        self._save_clipboard_indentation()
+
     # ---- Private methods (overrode by us) -----------------------------------
     def _handle_error(self, msg):
         """
@@ -806,13 +1006,32 @@ the sympy module (e.g. plot)
         super(ShellWidget, self)._handle_execute_input(msg)
         self.sig_remote_execute.emit()
 
+    def _process_execute_error(self, msg):
+        """
+        Display a message when using our installers to explain users
+        how to use modules that doesn't come with them.
+        """
+        super(ShellWidget, self)._process_execute_error(msg)
+        if self.show_modules_message:
+            error = msg['content']['traceback']
+            if any(['ModuleNotFoundError' in frame or 'ImportError' in frame
+                    for frame in error]):
+                self.append_html_message(
+                    _("It seems you're trying to use a module that doesn't "
+                      "come with our installer. Check "
+                      "<a href='{}'>this FAQ</a> in our docs to learn how "
+                      "to do this.").format(MODULES_FAQ_URL),
+                    before_prompt=True
+                )
+            self.show_modules_message = False
+
     #---- Qt methods ----------------------------------------------------------
     def focusInEvent(self, event):
         """Reimplement Qt method to send focus change notification"""
-        self.focus_changed.emit()
+        self.sig_focus_changed.emit()
         return super(ShellWidget, self).focusInEvent(event)
 
     def focusOutEvent(self, event):
         """Reimplement Qt method to send focus change notification"""
-        self.focus_changed.emit()
+        self.sig_focus_changed.emit()
         return super(ShellWidget, self).focusOutEvent(event)
