@@ -13,6 +13,7 @@ from qtpy.QtCore import Signal
 
 # Local imports
 from spyder.api.plugins import Plugins, SpyderDockablePlugin
+from spyder.api.plugin_registration.decorators import on_plugin_available
 from spyder.api.translations import get_translation
 from spyder.plugins.plots.widgets.main_widget import PlotsWidget
 
@@ -33,7 +34,7 @@ class Plots(SpyderDockablePlugin):
     CONF_FILE = False
     DISABLE_ACTIONS_WHEN_HIDDEN = False
 
-    # --- SpyderDockablePlugin API
+    # ---- SpyderDockablePlugin API
     # ------------------------------------------------------------------------
     def get_name(self):
         return _('Plots')
@@ -44,42 +45,34 @@ class Plots(SpyderDockablePlugin):
     def get_icon(self):
         return self.create_icon('hist')
 
-    def register(self):
+    def on_initialize(self):
+        # If a figure is loaded, raise the dockwidget the first time
+        # a plot is generated.
+        self.get_widget().sig_figure_loaded.connect(self._on_first_plot)
+
+    @on_plugin_available(plugin=Plugins.IPythonConsole)
+    def on_ipython_console_available(self):
         # Plugins
         ipyconsole = self.get_plugin(Plugins.IPythonConsole)
 
         # Signals
         ipyconsole.sig_shellwidget_changed.connect(self.set_shellwidget)
-        ipyconsole.sig_shellwidget_process_started.connect(
+        ipyconsole.sig_shellwidget_created.connect(
             self.add_shellwidget)
-        ipyconsole.sig_shellwidget_process_finished.connect(
+        ipyconsole.sig_shellwidget_deleted.connect(
             self.remove_shellwidget)
 
-        # If a figure is loaded raise the dockwidget but do not give focus
-        self.get_widget().sig_figure_loaded.connect(
-            lambda: self.switch_to_plugin(force_focus=False))
-
     def unregister(self):
         # Plugins
         ipyconsole = self.get_plugin(Plugins.IPythonConsole)
 
         # Signals
-        ipyconsole.sig_shellwidget_id_changed.disconnect(
-            self.set_shellwidget_from_id)
-        ipyconsole.sig_shellwidget_process_started.disconnect(
+        ipyconsole.sig_shellwidget_created.disconnect(
             self.add_shellwidget)
-        ipyconsole.sig_shellwidget_id_process_finished.disconnect(
-            self.remove_shellwidget_from_id)
+        ipyconsole.sig_shellwidget_deleted.connect(
+            self.remove_shellwidget)
 
-    def switch_to_plugin(self, force_focus=False):
-        # Only switch when inline plotting is muted. This avoids
-        # showing the plugin when users want to only see plots in
-        # the IPython console.
-        # Fixes spyder-ide/spyder#15467
-        if self.get_conf('mute_inline_plotting'):
-            super().switch_to_plugin(force_focus=force_focus)
-
-    # --- Public API
+    # ---- Public API
     # ------------------------------------------------------------------------
     def current_widget(self):
         """
@@ -127,3 +120,19 @@ class Plots(SpyderDockablePlugin):
             The shell widget.
         """
         self.get_widget().set_shellwidget(shellwidget)
+
+    # ---- Private API
+    # ------------------------------------------------------------------------
+    def _on_first_plot(self):
+        """Actions to execute after the first plot is generated."""
+        # Only switch when inline plotting is muted. This avoids
+        # showing the plugin when users want to only see plots in
+        # the IPython console.
+        # Fixes spyder-ide/spyder#15467
+        if self.get_conf('mute_inline_plotting'):
+            self.switch_to_plugin(force_focus=False)
+
+        # We only give raise to the plugin once per session, to let users
+        # know that plots are displayed in this plugin.
+        # Fixes spyder-ide/spyder#15705
+        self.get_widget().sig_figure_loaded.disconnect(self._on_first_plot)
